@@ -18,6 +18,7 @@ const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
 const OPEN_AND_CLOSE_TIME = 0.1;
+const FADE_IN_BUTTONS_TIME = 0.33;
 const FADE_OUT_DIALOG_TIME = 1.0;
 
 const State = {
@@ -34,10 +35,12 @@ function ModalDialog() {
 
 ModalDialog.prototype = {
     _init: function(params) {
-        params = Params.parse(params, { styleClass: null });
+        params = Params.parse(params, { shellReactive: false,
+                                        styleClass: null });
 
         this.state = State.CLOSED;
         this._hasModal = false;
+        this._shellReactive = params.shellReactive;
 
         this._group = new St.Group({ visible: false,
                                      x: 0,
@@ -53,26 +56,30 @@ ModalDialog.prototype = {
         this._actionKeys = {};
         this._group.connect('key-press-event', Lang.bind(this, this._onKeyPressEvent));
 
-        this._lightbox = new Lightbox.Lightbox(this._group,
-                                               { inhibitEvents: true });
-
         this._backgroundBin = new St.Bin();
-
         this._group.add_actor(this._backgroundBin);
-        this._lightbox.highlight(this._backgroundBin);
-
-        this._backgroundStack = new Shell.Stack();
-        this._backgroundBin.child = this._backgroundStack;
-
-        this._eventBlocker = new Clutter.Group({ reactive: true });
-        this._backgroundStack.add_actor(this._eventBlocker);
 
         this._dialogLayout = new St.BoxLayout({ style_class: 'modal-dialog',
                                                 vertical:    true });
         if (params.styleClass != null) {
             this._dialogLayout.add_style_class_name(params.styleClass);
         }
-        this._backgroundStack.add_actor(this._dialogLayout);
+
+        if (!this._shellReactive) {
+            this._lightbox = new Lightbox.Lightbox(this._group,
+                                                   { inhibitEvents: true });
+            this._lightbox.highlight(this._backgroundBin);
+
+            let stack = new Shell.Stack();
+            this._backgroundBin.child = stack;
+
+            this._eventBlocker = new Clutter.Group({ reactive: true });
+            stack.add_actor(this._eventBlocker);
+            stack.add_actor(this._dialogLayout);
+        } else {
+            this._backgroundBin.child = this._dialogLayout;
+        }
+
 
         this.contentLayout = new St.BoxLayout({ vertical: true });
         this._dialogLayout.add(this.contentLayout,
@@ -82,7 +89,6 @@ ModalDialog.prototype = {
                                  y_align: St.Align.START });
 
         this._buttonLayout = new St.BoxLayout({ style_class: 'modal-dialog-button-box',
-                                                opacity:     220,
                                                 vertical:    false });
         this._dialogLayout.add(this._buttonLayout,
                                { expand:  true,
@@ -94,7 +100,13 @@ ModalDialog.prototype = {
         this._savedKeyFocus = null;
     },
 
+    destroy: function() {
+        this._group.destroy();
+    },
+
     setButtons: function(buttons) {
+        let hadChildren = this._buttonLayout.get_children() > 0;
+
         this._buttonLayout.destroy_children();
         this._actionKeys = {};
 
@@ -105,10 +117,10 @@ ModalDialog.prototype = {
             let action = buttonInfo['action'];
             let key = buttonInfo['key'];
 
-            let button = new St.Button({ style_class: 'modal-dialog-button',
-                                         reactive:    true,
-                                         can_focus:   true,
-                                         label:       label });
+            buttonInfo.button = new St.Button({ style_class: 'modal-dialog-button',
+                                                reactive:    true,
+                                                can_focus:   true,
+                                                label:       label });
 
             let x_alignment;
             if (buttons.length == 1)
@@ -120,20 +132,36 @@ ModalDialog.prototype = {
             else
                 x_alignment = St.Align.MIDDLE;
 
-            this._initialKeyFocus = button;
-            this._buttonLayout.add(button,
+            this._initialKeyFocus = buttonInfo.button;
+            this._buttonLayout.add(buttonInfo.button,
                                    { expand: true,
                                      x_fill: false,
                                      y_fill: false,
                                      x_align: x_alignment,
                                      y_align: St.Align.MIDDLE });
 
-            button.connect('clicked', action);
+            buttonInfo.button.connect('clicked', action);
 
             if (key)
                 this._actionKeys[key] = action;
             i++;
         }
+
+        // Fade in buttons if there weren't any before
+        if (!hadChildren && i > 0) {
+            this._buttonLayout.opacity = 0;
+            Tweener.addTween(this._buttonLayout,
+                             { opacity: 255,
+                               time: FADE_IN_BUTTONS_TIME,
+                               transition: 'easeOutQuad',
+                               onComplete: Lang.bind(this, function() {
+                                   this.emit('buttons-set');
+                               })
+                             });
+        } else {
+            this.emit('buttons-set');
+        }
+
     },
 
     _onKeyPressEvent: function(object, keyPressEvent) {
@@ -149,7 +177,7 @@ ModalDialog.prototype = {
     },
 
     _fadeOpen: function() {
-        let monitor = global.get_focus_monitor();
+        let monitor = Main.layoutManager.focusMonitor;
 
         this._backgroundBin.set_position(monitor.x, monitor.y);
         this._backgroundBin.set_size(monitor.width, monitor.height);
@@ -157,7 +185,8 @@ ModalDialog.prototype = {
         this.state = State.OPENING;
 
         this._dialogLayout.opacity = 255;
-        this._lightbox.show();
+        if (this._lightbox)
+            this._lightbox.show();
         this._group.opacity = 0;
         this._group.show();
         Tweener.addTween(this._group,
@@ -223,7 +252,8 @@ ModalDialog.prototype = {
         global.gdk_screen.get_display().sync();
         this._hasModal = false;
 
-        this._eventBlocker.raise_top();
+        if (!this._shellReactive)
+            this._eventBlocker.raise_top();
     },
 
     pushModal: function (timestamp) {
@@ -239,7 +269,8 @@ ModalDialog.prototype = {
         } else
             this._initialKeyFocus.grab_key_focus();
 
-        this._eventBlocker.lower_bottom();
+        if (!this._shellReactive)
+            this._eventBlocker.lower_bottom();
         return true;
     },
 
